/*
  Copyright 2018-2020, Barcelona Supercomputing Center (BSC), Spain
  Copyright 2015-2020, Johannes Gutenberg Universitaet Mainz, Germany

  This software was partially supported by the
  EC H2020 funded project NEXTGenIO (Project ID: 671951, www.nextgenio.eu).

  This software was partially supported by the
  ADA-FS project under the SPPEXA project funded by the DFG.

  SPDX-License-Identifier: MIT
*/

#include <hermes.hpp>
#include <client/rpc/rpc_types.hpp>

//==============================================================================
// register request types so that they can be used by users and the engine
//
void hermes::detail::register_user_request_types(uint32_t provider_id) {
    if (provider_id == 0) {
        (void) registered_requests().add<gkfs::rpc::fs_config>(provider_id);
        (void) registered_requests().add<gkfs::rpc::create>(provider_id);
        (void) registered_requests().add<gkfs::rpc::stat>(provider_id);
        (void) registered_requests().add<gkfs::rpc::remove>(provider_id);
        (void) registered_requests().add<gkfs::rpc::decr_size>(provider_id);
        (void) registered_requests().add<gkfs::rpc::update_metadentry>(provider_id);
        (void) registered_requests().add<gkfs::rpc::get_metadentry_size>(provider_id);
        (void) registered_requests().add<gkfs::rpc::update_metadentry_size>(provider_id);

#ifdef HAS_SYMLINKS
        (void) registered_requests().add<gkfs::rpc::mk_symlink>(provider_id);
#endif // HAS_SYMLINKS

        (void) registered_requests().add<gkfs::rpc::write_data>(provider_id);
        (void) registered_requests().add<gkfs::rpc::read_data>(provider_id);
        (void) registered_requests().add<gkfs::rpc::trunc_data>(provider_id);
        (void) registered_requests().add<gkfs::rpc::get_dirents>(provider_id);
        (void) registered_requests().add<gkfs::rpc::chunk_stat>(provider_id);
    } else {
        (void) registered_requests().add<gkfs::rpc::write_data_proxy>(provider_id);
        (void) registered_requests().add<gkfs::rpc::read_data_proxy>(provider_id);
        (void) registered_requests().add<gkfs::rpc::create_proxy>(provider_id);
        (void) registered_requests().add<gkfs::rpc::stat_proxy>(provider_id);
    }

}