/*
  Copyright 2018-2020, Barcelona Supercomputing Center (BSC), Spain
  Copyright 2015-2020, Johannes Gutenberg Universitaet Mainz, Germany

  This software was partially supported by the
  EC H2020 funded project NEXTGenIO (Project ID: 671951, www.nextgenio.eu).

  This software was partially supported by the
  ADA-FS project under the SPPEXA project funded by the DFG.

  SPDX-License-Identifier: MIT
*/

#include <proxy/proxy.hpp>
#include <proxy/rpc/rpc_defs.hpp>
#include <proxy/rpc/forward_metadata.hpp>
#include <proxy/rpc/rpc_util.hpp>

#include <global/rpc/rpc_types.hpp>

static hg_return_t proxy_rpc_srv_create(hg_handle_t handle) {
    rpc_mk_node_in_t client_in{};
    rpc_err_out_t client_out{};

    auto ret = margo_get_input(handle, &client_in);
    if (ret != HG_SUCCESS) {
        PROXY_DATA->log()->error("{}() Failed to retrieve input from handle", __func__);
        return gkfs::rpc::cleanup_respond(&handle, &client_in, &client_out);
    }
    PROXY_DATA->log()->debug("{}() Got RPC with path '{}'", __func__, client_in.path);

    client_out.err = gkfs::rpc::forward_create(client_in.path, client_in.mode);

    PROXY_DATA->log()->debug("{}() Sending output err '{}'", __func__, client_out.err);
    return gkfs::rpc::cleanup_respond(&handle, &client_in, &client_out);
}

DEFINE_MARGO_RPC_HANDLER(proxy_rpc_srv_create)

static hg_return_t proxy_rpc_srv_stat(hg_handle_t handle) {
    rpc_path_only_in_t client_in{};
    rpc_stat_out_t client_out{};

    auto ret = margo_get_input(handle, &client_in);
    if (ret != HG_SUCCESS) {
        PROXY_DATA->log()->error("{}() Failed to retrieve input from handle", __func__);
        return gkfs::rpc::cleanup_respond(&handle, &client_in, &client_out);
    }
    PROXY_DATA->log()->debug("{}() Got RPC with path '{}'", __func__, client_in.path);

    auto out = gkfs::rpc::forward_stat(client_in.path);
    client_out.err = out.first;
    client_out.db_val = out.second.c_str();

    PROXY_DATA->log()->debug("{}() Sending output err '{}'", __func__, client_out.err);
    return gkfs::rpc::cleanup_respond(&handle, &client_in, &client_out);
}

DEFINE_MARGO_RPC_HANDLER(proxy_rpc_srv_stat)

static hg_return_t proxy_rpc_srv_remove(hg_handle_t handle) {
    return HG_SUCCESS;
}

DEFINE_MARGO_RPC_HANDLER(proxy_rpc_srv_remove)
